package com.glarimy.atlas.biz;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;

import com.glarimy.atlas.api.Atlas;
import com.glarimy.atlas.api.AtlasException;
import com.glarimy.atlas.api.Country;
import com.glarimy.atlas.api.CountryNotFoundException;
import com.glarimy.atlas.api.DuplicateCountryException;
import com.glarimy.atlas.api.InvalidCountryException;
import com.glarimy.atlas.api.InvalidKeyException;

public class AtlasDBImpl implements Atlas {
	private Map<String, Country> cache;
	private static AtlasDBImpl instance;

	private AtlasDBImpl() throws Exception {
		cache = new HashMap<String, Country>();
		try {
			ObjectInputStream in = new ObjectInputStream(new FileInputStream(
					"atlas.data"));
			Object o = null;
			while ((o = in.readObject()) != null) {
				Country country = (Country) o;
				cache.put(country.getName(), country);
			}
			in.close();
		} catch (Exception e) {
			System.out.println("Not data found! Starting from zero");
		}
	}

	public static synchronized AtlasDBImpl getInstance() throws Exception {
		if (instance == null)
			instance = new AtlasDBImpl();
		return instance;
	}

	@Override
	public Country search(String name) throws InvalidKeyException,
			CountryNotFoundException, AtlasException {
		if (name == null || name.trim().length() == 0)
			throw new InvalidKeyException("Invalid country name!");
		Country country = cache.get(name);
		if (country == null)
			throw new CountryNotFoundException(name + " is not found!");
		return country;
	}

	@Override
	public void add(Country country) throws InvalidCountryException,
			DuplicateCountryException, AtlasException {
		ObjectOutputStream out = null;
		try {
			out = new ObjectOutputStream(new FileOutputStream("atlas.data",
					true));
			out.writeObject(country);
			cache.put(country.getName(), country);
		} catch (Exception e) {
			throw new AtlasException(e);
		} finally {
			try {
				out.close();
			} catch (IOException e) {
			}
		}
	}
}
