package com.glarimy.atlas.biz;

import com.glarimy.atlas.api.Atlas;
import com.glarimy.atlas.api.AtlasException;
import com.glarimy.atlas.api.Country;
import com.glarimy.atlas.api.CountryNotFoundException;
import com.glarimy.atlas.api.DuplicateCountryException;
import com.glarimy.atlas.api.InvalidCountryException;
import com.glarimy.atlas.api.InvalidKeyException;

public class AtlasImpl implements Atlas {
	private static AtlasImpl instance;
	private CountryDAO dao;

	private AtlasImpl() throws Exception {
		dao = new CountryDAO();
	}

	public static synchronized AtlasImpl getInstance() throws Exception {
		if (instance == null)
			instance = new AtlasImpl();
		return instance;
	}

	@Override
	public Country search(String name) throws InvalidKeyException,
			CountryNotFoundException, AtlasException {
		if (name == null || name.trim().length() == 0)
			throw new InvalidKeyException("Invalid country name!");
		Country country = dao.find(name);
		if (country == null)
			throw new CountryNotFoundException(name + " is not found!");
		return country;
	}

	@Override
	public void add(Country country) throws InvalidCountryException,
			DuplicateCountryException, AtlasException {
		dao.persist(country);
	}
}
