package com.glarimy.bank.biz;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import com.glarimy.bank.api.Bank;
import com.glarimy.bank.api.Person;
import com.glarimy.bank.api.Transaction;

public class GlarimyBank implements Bank {
	private HashMap<Integer, Account> ledger;

	public GlarimyBank() {
		ledger = new HashMap<Integer, Account>();
	}

	@Override
	public int openAccount(Person customerData) {
		if (customerData.getName() == null
				|| customerData.getName().trim().length() == 0
				|| customerData.getCity() == null
				|| customerData.getCity().trim().length() == 0
				|| customerData.getPhone() <= 0) {
			System.out.println("Invalid customer data. Denied!");
			return -1;
		}

		Account account = new Account(ledger.size(), 0, true, customerData);
		ledger.put(account.accountNumber, account);
		Transaction transaction = new Transaction();
		transaction.setBalance(account.accountBalance);
		transaction.setType(1);
		transaction.setDescription("Account Opened");
		transaction.setDate(new Date());
		account.history.add(transaction);

		return account.accountNumber;
	}

	@Override
	public double deposit(int number, double amount) {
		if (amount <= 0) {
			System.out.println("Invalid Amount!");
			return -1;
		}

		Account account = ledger.get(number);
		if (account == null) {
			System.out.println("Account Not Found!");
			return -1;
		}

		if (account.accountStatus == false) {
			System.out.println("Closed Account!");
			return -3;
		}
		account.accountBalance += amount;
		Transaction transaction = new Transaction();
		transaction.setAmount(amount);
		transaction.setBalance(account.accountBalance);
		transaction.setType(2);
		transaction.setDescription("Cash deposit");
		transaction.setDate(new Date());
		account.history.add(transaction);
		return account.accountBalance;

	}

	@Override
	public double withdraw(int number, double amount) {
		if (amount <= 0) {
			System.out.println("Invalid Amount!");
			return -1;
		}

		Account account = ledger.get(number);
		if (account == null) {
			System.out.println("Account Not Found!");
			return -1;
		}

		if (account.accountStatus == false) {
			System.out.println("Closed Account!");
			return -3;
		}

		if (account.accountBalance < amount) {
			System.out.println("Insufficient Balance!");
			return -4;
		}
		account.accountBalance -= amount;
		Transaction transaction = new Transaction();
		transaction.setAmount(amount);
		transaction.setBalance(account.accountBalance);
		transaction.setType(3);
		transaction.setDescription("Cash withdrawl");
		transaction.setDate(new Date());
		account.history.add(transaction);
		return account.accountBalance;
	}

	@Override
	public boolean close(int number) {
		Account account = ledger.get(number);
		if (account == null) {
			System.out.println("Account Not Found!");
			return false;
		}

		if (account.accountStatus == false) {
			System.out.println("Account Already Closed!");
			return true;
		}
		Transaction transaction = new Transaction();
		transaction.setAmount(account.accountBalance);
		transaction.setBalance(0);
		transaction.setType(4);
		transaction.setDescription("Account Closed");
		transaction.setDate(new Date());
		account.history.add(transaction);
		account.accountBalance = 0;
		account.accountStatus = false;
		return true;

	}

	@Override
	public ArrayList<Transaction> history(int number) {
		Account account = ledger.get(number);
		if (account == null) {
			System.out.println("Account Not Found!");
			return null;
		}
		return account.history;

	}
}