package com.glarimy.bank.biz;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import com.glarimy.bank.api.AccountNotFoundException;
import com.glarimy.bank.api.Bank;
import com.glarimy.bank.api.BankException;
import com.glarimy.bank.api.InsufficientBalanceException;
import com.glarimy.bank.api.InvalidAmountException;
import com.glarimy.bank.api.InvalidDataException;
import com.glarimy.bank.api.InvalidStateException;
import com.glarimy.bank.api.Person;
import com.glarimy.bank.api.Transaction;

public class GlarimyBank implements Bank {
	private HashMap<Integer, Account> ledger;

	public GlarimyBank() {
		ledger = new HashMap<Integer, Account>();
	}

	@Override
	public int openAccount(Person customerData) throws InvalidDataException,
			BankException {
		if (customerData.getName() == null
				|| customerData.getName().trim().length() == 0
				|| customerData.getCity() == null
				|| customerData.getCity().trim().length() == 0
				|| customerData.getPhone() <= 0) {
			throw new InvalidDataException("Invalid customer data. Denied!");
		}

		Account account = new Account(ledger.size(), 0, true, customerData);
		ledger.put(account.accountNumber, account);
		Transaction transaction = new Transaction();
		transaction.setBalance(account.accountBalance);
		transaction.setType(1);
		transaction.setDescription("Account Opened");
		transaction.setDate(new Date());
		account.history.add(transaction);

		return account.accountNumber;
	}

	@Override
	public double deposit(int number, double amount)
			throws AccountNotFoundException, InvalidStateException,
			InvalidAmountException, BankException {
		if (amount <= 0)
			throw new InvalidAmountException("Invalid Amount!");

		Account account = ledger.get(number);
		if (account == null)
			throw new AccountNotFoundException("Account Not Found!");

		if (account.accountStatus == false)
			throw new InvalidStateException("Closed Account!");

		account.accountBalance += amount;
		Transaction transaction = new Transaction();
		transaction.setAmount(amount);
		transaction.setBalance(account.accountBalance);
		transaction.setType(2);
		transaction.setDescription("Cash deposit");
		transaction.setDate(new Date());
		account.history.add(transaction);
		return account.accountBalance;

	}

	@Override
	public double withdraw(int number, double amount)
			throws AccountNotFoundException, InvalidAmountException,
			InvalidStateException, BankException {
		if (amount <= 0)
			throw new InvalidAmountException("Invalid Amount!");

		Account account = ledger.get(number);
		if (account == null)
			throw new AccountNotFoundException("Account Not Found!");

		if (account.accountStatus == false)
			throw new InvalidStateException("Closed Account!");

		if (account.accountBalance < amount)
			throw new InsufficientBalanceException("Insufficient Balance!");

		account.accountBalance -= amount;
		Transaction transaction = new Transaction();
		transaction.setAmount(amount);
		transaction.setBalance(account.accountBalance);
		transaction.setType(3);
		transaction.setDescription("Cash withdrawl");
		transaction.setDate(new Date());
		account.history.add(transaction);
		return account.accountBalance;
	}

	@Override
	public boolean close(int number) throws AccountNotFoundException,
			InvalidStateException, BankException {
		Account account = ledger.get(number);
		if (account == null)
			throw new AccountNotFoundException("Account Not Found!");

		if (account.accountStatus == false)
			throw new InvalidStateException("Closed Account!");
		Transaction transaction = new Transaction();
		transaction.setAmount(account.accountBalance);
		transaction.setBalance(0);
		transaction.setType(4);
		transaction.setDescription("Account Closed");
		transaction.setDate(new Date());
		account.history.add(transaction);
		account.accountBalance = 0;
		account.accountStatus = false;
		return true;

	}

	@Override
	public ArrayList<Transaction> history(int number)
			throws AccountNotFoundException, BankException {
		Account account = ledger.get(number);
		if (account == null)
			throw new AccountNotFoundException("Account Not Found!");
		return account.history;

	}
}