package com.glarimy.bank.biz;

import java.util.Date;

import com.glarimy.bank.api.Bank;
import com.glarimy.bank.api.Person;
import com.glarimy.bank.api.Transaction;

public class GlarimyBank implements Bank {
	private Account[] ledger;
	private int count;
	
	public GlarimyBank() {
		ledger = new Account[10];
	}

	@Override
	public int openAccount(Person customerData) {
		if (customerData.getName() == null
				|| customerData.getName().trim().length() == 0
				|| customerData.getCity() == null
				|| customerData.getCity().trim().length() == 0
				|| customerData.getPhone() <= 0) {
			System.out.println("Invalid customer data. Denied!");
			return -1;
		}

		if (count == 10) {
			System.out.println("Internal Error!");
			return -2;
		}

		Account account = new Account(count, 0, true, customerData);
		ledger[count++] = account;
		Transaction transaction = new Transaction();
		transaction.setBalance(account.accountBalance);
		transaction.setType(1);
		transaction.setDescription("Account Opened");
		transaction.setDate(new Date());
		account.history[account.numberOfTransactions++] = transaction;

		return account.accountNumber;
	}

	@Override
	public double deposit(int number, double amount) {
		if (amount <= 0) {
			System.out.println("Invalid Amount!");
			return -1;
		}
		for (Account account : ledger)
			if (account.accountNumber == number) {
				if(account.accountStatus == false){
					System.out.println("Closed Account!");
					return -3;					
				}
				if (account.numberOfTransactions == 10) {
					System.out.println("Exceeded Transaction Limit!");
					return -3;
				}
				account.accountBalance += amount;
				Transaction transaction = new Transaction();
				transaction.setAmount(amount);
				transaction.setBalance(account.accountBalance);
				transaction.setType(2);
				transaction.setDescription("Cash deposit");
				transaction.setDate(new Date());
				account.history[account.numberOfTransactions++] = transaction;
				return account.accountBalance;
			}
		System.out.println("Account Not Found!");
		return -1;
	}

	@Override
	public double withdraw(int number, double amount) {
		if (amount <= 0) {
			System.out.println("Invalid Amount!");
			return -1;
		}
		for (Account account : ledger)
			if (account.accountNumber == number) {
				if(account.accountStatus == false){
					System.out.println("Closed Account!");
					return -3;					
				}
				
				if (account.numberOfTransactions == 10) {
					System.out.println("Exceeded Transaction Limit!");
					return -3;
				}

				if (account.accountBalance < amount) {
					System.out.println("Insufficient Balance!");
					return -4;
				}
				account.accountBalance -= amount;
				Transaction transaction = new Transaction();
				transaction.setAmount(amount);
				transaction.setBalance(account.accountBalance);
				transaction.setType(3);
				transaction.setDescription("Cash withdrawl");
				transaction.setDate(new Date());
				account.history[account.numberOfTransactions++] = transaction;
				return account.accountBalance;
			}
		System.out.println("Account Not Found!");
		return -1;
	}

	@Override
	public boolean close(int number) {
		for (Account account : ledger)
			if (account.accountNumber == number) {
				if (account.accountStatus == false) {
					System.out.println("Account Already Closed!");
					return true;
				}
				Transaction transaction = new Transaction();
				transaction.setAmount(account.accountBalance);
				transaction.setBalance(0);
				transaction.setType(4);
				transaction.setDescription("Account Closed");
				transaction.setDate(new Date());
				account.history[account.numberOfTransactions++] = transaction;
				account.accountBalance = 0;
				account.accountStatus = false;
				return true;
			}
		System.out.println("Account Not Found!");
		return false;

	}

	@Override
	public Transaction[] history(int number) {
		for (Account account : ledger)
			if (account.accountNumber == number) {
				if (account.numberOfTransactions == 10)
					return account.history;
				else {
					Transaction[] history = new Transaction[account.numberOfTransactions];
					for (int i = 0; i < account.numberOfTransactions; i++)
						history[i] = account.history[i];
					return history;
				}
			}
		System.out.println("Account Not Found!");
		return null;
	}
}