package com.glarimy.java.lib.biz;

import com.glarimy.java.lib.api.Book;
import com.glarimy.java.lib.api.BookNotFoundException;
import com.glarimy.java.lib.api.DuplicateException;
import com.glarimy.java.lib.api.InvalidBookException;
import com.glarimy.java.lib.api.InvalidKeyException;
import com.glarimy.java.lib.api.Library;
import com.glarimy.java.lib.api.LibraryException;
import com.glarimy.java.lib.api.NoSpaceException;

public class RobustLibrary implements Library {
	private Book[] rack;
	private int count;

	public RobustLibrary() {
		rack = new Book[10];
	}

	@Override
	public void insert(Book book) throws NoSpaceException, DuplicateException,
			InvalidBookException, LibraryException {
		if (book == null || book.getTitle().trim().length() == 0
				|| book.getIsbn() < 1)
			throw new InvalidBookException();
		if (count == 10)
			throw new NoSpaceException();
		for (int i = 0; i < count; i++)
			if (book.getIsbn() == rack[i].getIsbn())
				throw new DuplicateException();
		if (count < 10)
			rack[count++] = book;

	}

	@Override
	public Book search(int isbn) throws InvalidKeyException,
			BookNotFoundException, LibraryException {
		if (isbn < 1)
			throw new InvalidKeyException();
		for (int i = 0; i < count; i++)
			if (rack[i].getIsbn() == isbn)
				return rack[i];
		throw new BookNotFoundException();
	}

	@Override
	public Book[] search(String title) throws InvalidKeyException,
			BookNotFoundException, LibraryException {
		if (title == null || title.trim().length() == 0)
			throw new InvalidKeyException();
		Book[] results = new Book[count];
		int index = 0;
		for (int i = 0; i < count; i++)
			if (rack[i].getTitle().contains(title))
				results[index++] = rack[i];
		if (index == 0)
			throw new BookNotFoundException();
		if (index == rack.length)
			return results;
		Book[] matches = new Book[index];
		for (int i = 0; i < index; i++)
			matches[i] = results[i];
		return matches;
	}

}
