package com.glarimy.threads;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

class LazyCounter {
    private int value;
    private Lock lock;

    public LazyCounter() {
        lock = new ReentrantLock();
    }

    public void count(long delay) {
        String name = Thread.currentThread().getName();
        try {
            if (lock.tryLock(1, TimeUnit.SECONDS) == false) {
                System.out.println(name + " : Cant wait. Giving up ...");
                return;
            }
            for (int i = 0; i < 5; i++, value++) {
                System.out.println(name + " : " + value);
                Thread.sleep(delay);
            }
        } catch (InterruptedException e1) {
            System.out.println(name + " : Backing off ...");
        }
        lock.unlock();
    }
}

class LazyCountingThread extends Thread {
    private LazyCounter counter;
    private long delay;

    public LazyCountingThread(LazyCounter counter, long delay) {
        this.counter = counter;
        this.delay = delay;
    }

    @Override
    public void run() {
        counter.count(delay);
    }

    public static void main(String[] args) {
        LazyCounter counter = new LazyCounter();
        LazyCountingThread one = new LazyCountingThread(counter, 2000);
        LazyCountingThread two = new LazyCountingThread(counter, 5000);
        one.start();
        two.start();
    }
}