package com.glarimy.threads;

class CommonCounter {
	private int value;
	private boolean okToRead = false;

	public synchronized void read() {
		String name = Thread.currentThread().getName();
		try {
			while (!okToRead) {
				System.out.format("%s : waiting for the writer... %n", name);
				wait();
				System.out.format("%s : received notification... %n", name);
			}

			Thread.sleep(1000);
			okToRead = false;
			System.out.format("%s : reading the value: %d and notifying %n",
					name, value);
			notifyAll();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public synchronized void write() {
		String name = Thread.currentThread().getName();
		try {
			while (okToRead) {
				System.out.format("%s : waiting for the reader... %n", name);
				wait();
				System.out.format("%s : received notification... %n", name);
			}
			value++;
			Thread.sleep(5000);
			okToRead = true;
			System.out.format("%s : written value and notifying ... %n", name);
			notifyAll();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}

class Reader extends Thread {
	private CommonCounter counter;

	public Reader(CommonCounter counter) {
		this.counter = counter;
		this.setName("Reader");
	}

	@Override
	public void run() {
		while (true)
			counter.read();
	}
}

class Writer extends Thread {
	private CommonCounter counter;

	public Writer(CommonCounter counter) {
		this.counter = counter;
		this.setName("Writer");
	}

	@Override
	public void run() {
		while (true)
			counter.write();
	}
}

public class CountingThread {

	public static void main(String[] args) {
		CommonCounter counter = new CommonCounter();
		Reader reader = new Reader(counter);
		Writer writer = new Writer(counter);
		writer.start();
		reader.start();
	}
}